use diesel::result::Error;
use rocket::http::Status;
use rocket::response::status;
use rocket_contrib::json::Json;
use rocket_contrib::templates::Template;
use std::collections::HashMap;

use crate::connection::DbConn;
use crate::gists;
use crate::gists::{Gist, InsertableGist};

fn error_response(error: Error) -> Status {
    match error {
        Error::NotFound => Status::NotFound,
        _ => Status::InternalServerError,
    }
}

#[get("/")]
pub fn index(connection: DbConn) -> Result<Json<Vec<Gist>>, Status> {
    gists::all(&connection)
        .map(|gists| Json(gists))
        .map_err(|error| error_response(error))
}

#[get("/gists/<id>")]
pub fn show_gist(id: i32, connection: DbConn) -> Template {
    let gist = match gists::get(&connection, id) {
        Ok(gist) => Some(gist),
        Err(_) => None,
    };
    let mut context = HashMap::new();
    context.insert("gist", gist);
    Template::render("gists/show", &context)
}

#[post("/api/gists", format = "application/json", data = "<gist>")]
pub fn create_gist<'a>(
    gist: Json<InsertableGist>,
    connection: DbConn,
) -> Result<status::Created<Json<Gist>>, Status> {
    gists::insert(gist.into_inner(), &connection)
        .map(|gist| status::Created(String::from(""), Some(Json(gist))))
        .map_err(|error| error_response(error))
}
